# pylint: disable=missing-docstring,redefined-outer-name
import os
import tempfile
from collections.abc import Mapping, MutableMapping
from pathlib import Path
from unittest import mock

from unittest_fixtures import FixtureContext, Fixtures, fixture


@fixture()
def environ(
    _fixtures: Fixtures, *, environ: Mapping[str, str] | None = None
) -> FixtureContext[MutableMapping[str, str]]:
    environ = environ or {}

    with mock.patch.dict(os.environ, clear=True):
        os.environ.update(environ)
        yield os.environ


@fixture()
def tmpdir(_fixtures: Fixtures) -> FixtureContext[Path]:
    with tempfile.TemporaryDirectory() as tempdir:
        yield Path(tempdir)



# pylint: disable=missing-docstring
from unittest import TestCase, mock

from gbp_webhook_tts import handlers, utils


@mock.patch.object(handlers.utils, "acquire_sound_file")
@mock.patch.object(handlers.sp, "Popen")
class BuildPulledTests(TestCase):
    def test(self, popen_cls: mock.Mock, acquire_sound_file: mock.Mock) -> None:
        event = {"name": "build_pulled", "machine": "babette", "data": {}}
        handlers.build_pulled(event)
        player = utils.get_sound_player()
        sound_file = acquire_sound_file.return_value

        acquire_sound_file.assert_called_once_with(event)
        popen_cls.assert_called_once_with([*player, str(sound_file)])



# pylint: disable=missing-docstring,unused-argument
from pathlib import Path
from unittest import TestCase, mock

from unittest_fixtures import Fixtures, given

from gbp_webhook_tts import utils

EVENT = {"name": "build_pulled", "machine": "babette", "data": {}}


@given("tmpdir")
@mock.patch.object(utils, "event_to_speech")
@mock.patch.object(utils.platformdirs, "user_cache_path")
class AcquireSoundFileTests(TestCase):
    def test(
        self, user_cache_path: mock.Mock, event_to_speech: mock.Mock, fixtures: Fixtures
    ) -> None:
        tmpdir = fixtures.tmpdir
        user_cache_path.return_value = tmpdir
        event_to_speech.return_value = b"test"
        path = utils.acquire_sound_file(EVENT)

        self.assertEqual(tmpdir / "tts" / "babette.mp3", path)
        self.assertTrue(path.parent.is_dir())
        self.assertEqual(path.read_bytes(), b"test")

    def test_makes_path(
        self, user_cache_path: mock.Mock, event_to_speech: mock.Mock, fixtures: Fixtures
    ) -> None:
        tmpdir = fixtures.tmpdir
        user_cache_path.return_value = tmpdir / "foo"
        event_to_speech.return_value = b"test"
        path = utils.acquire_sound_file(EVENT)

        self.assertEqual(tmpdir / "foo" / "tts" / "babette.mp3", path)
        self.assertTrue(path.parent.is_dir())
        self.assertEqual(path.read_bytes(), b"test")


@mock.patch.object(utils.platformdirs, "user_cache_path")
class EventToPathTests(TestCase):

    def test(self, user_cache_path: mock.Mock) -> None:
        user_cache_path.return_value = Path("/dev/null")
        path = utils.event_to_path(EVENT)

        self.assertEqual(Path("/dev/null/tts/babette.mp3"), path)


@mock.patch.object(utils.boto3, "Session")
class EventToSpeechTests(TestCase):

    def test(self, session_cls: mock.Mock) -> None:
        text = utils.get_speech_text_for_machine("babette")
        audio = utils.event_to_speech(EVENT)

        session = session_cls.return_value
        session.client.assert_called_once_with("polly")
        polly = session.client.return_value
        polly.synthesize_speech.assert_called_once_with(
            VoiceId="Ivy", OutputFormat="mp3", Text=text, TextType="ssml"
        )
        speech = polly.synthesize_speech.return_value
        self.assertEqual(speech["AudioStream"].read.return_value, audio)


@given("environ")
class GetSpeechTextForMachineTests(TestCase):
    def test(self, fixtures: Fixtures) -> None:
        environ = fixtures.environ
        environ["GBP_WEBHOOK_TTS_PHONETIC_KDE_DESKTOP"] = "foobar"

        self.assertEqual(
            ssml("foobar", 0), utils.get_speech_text_for_machine("kde-desktop")
        )

    def test_default(self, fixtures: Fixtures) -> None:
        self.assertEqual(
            ssml("kde desktop", 0), utils.get_speech_text_for_machine("kde-desktop")
        )

    def test_with_delay(self, fixtures: Fixtures) -> None:
        environ = fixtures.environ
        environ["GBP_WEBHOOK_TTS_DELAY"] = "0.8"

        self.assertEqual(
            ssml("kde desktop", 0.8), utils.get_speech_text_for_machine("kde-desktop")
        )


@given("environ")
class MapMachineToTextTests(TestCase):
    def test(self, fixtures: Fixtures) -> None:
        environ = fixtures.environ
        environ["GBP_WEBHOOK_TTS_PHONETIC_KDE_DESKTOP"] = "foobar"

        self.assertEqual("foobar", utils.map_machine_to_text("kde-desktop"))

    def test_default(self, fixtures: Fixtures) -> None:
        self.assertEqual(None, utils.map_machine_to_text("kde-desktop"))


@given("environ")
class GetSoundFileTests(TestCase):
    def test(self, fixtures: Fixtures) -> None:
        environ = fixtures.environ
        environ["GBP_WEBHOOK_PLAYSOUND_PLAYER"] = "blahblahblah blah"

        self.assertEqual(["blahblahblah", "blah"], utils.get_sound_player())

    def test_default(self, fixtures: Fixtures) -> None:
        self.assertEqual(["pw-play"], utils.get_sound_player())


def ssml(text: str, delay: float) -> str:
    return f"""\
<speak>
  <break time="{delay}s"/>
  {text}
</speak>"""



# pylint: disable=missing-docstring
from unittest import TestCase

from gbp_webhook_tts import templates


class LoadTemplateTests(TestCase):
    def test(self) -> None:
        template = templates.load_template("build_pulled.ssml")

        self.assertEqual("build_pulled.ssml", template.name)
        assert template.filename
        self.assertTrue(template.filename.endswith("/build_pulled.ssml"))

    def test_not_found(self) -> None:
        with self.assertRaises(templates.TemplateNotFoundError):
            templates.load_template("bogus")


class RenderTemplateTests(TestCase):
    def test(self) -> None:
        context = {"machine": "babette", "delay": 0.8}
        template = templates.load_template("build_pulled.ssml")

        text = templates.render_template(template, context)

        expected = """\
<speak>
  <break time="0.8s"/>
  babette
</speak>"""
        self.assertEqual(expected, text)



"""gbp-webhook-tts utils"""

import os
from pathlib import Path
from typing import Any

import boto3
import platformdirs

from gbp_webhook_tts.templates import load_template, render_template

environ = os.environ


def acquire_sound_file(event: dict[str, Any]) -> Path:
    """Acquire the audio file needed for the event and return the path"""
    path: Path = event_to_path(event)
    if not path.exists():
        audio = event_to_speech(event)
        path.parent.mkdir(parents=True, exist_ok=True)
        path.write_bytes(audio)
    return path


def event_to_path(event: dict[str, Any]) -> Path:
    """Given the path return the pathname of the audio file"""
    machine = event["machine"]

    return platformdirs.user_cache_path("gbp-webhook") / "tts" / f"{machine}.mp3"


def event_to_speech(event: dict[str, Any]) -> bytes:
    """Return .mp3 audio for the given event"""
    text = get_speech_text_for_machine(event["machine"])
    polly = boto3.Session().client("polly")
    response = polly.synthesize_speech(
        VoiceId="Ivy", OutputFormat="mp3", Text=text, TextType="ssml"
    )

    return response["AudioStream"].read()


def get_speech_text_for_machine(machine: str) -> str:
    """Given the machine name, return the text for speech"""
    template = load_template("build_pulled.ssml")
    context = {
        "machine": map_machine_to_text(machine) or machine.replace("-", " "),
        "delay": environ.get("GBP_WEBHOOK_TTS_DELAY", "0"),
    }
    return render_template(template, context)


def map_machine_to_text(machine: str) -> str | None:
    """Return the phonetic pronunciation for machine if defined

    For example if the machine is "kde-desktop", will return the value of the
    environment variable `GBP_WEBHOOK_TTS_PHONETIC_KDE_DESKTOP`
    """
    return environ.get(f"GBP_WEBHOOK_TTS_PHONETIC_{machine.replace('-', '_').upper()}")


def get_sound_player() -> list[str]:
    """Return the sound player executable"""
    return os.environ.get("GBP_WEBHOOK_PLAYSOUND_PLAYER", "pw-play").split()



"""gbp-webhook handler"""

import subprocess as sp
from typing import Any

from . import utils


def build_pulled(event: dict[str, Any]) -> None:
    """build_pulled event handler"""
    with sp.Popen([*utils.get_sound_player(), str(utils.acquire_sound_file(event))]):
        pass



"""GBP Notifications Template handling"""

import typing as t

import jinja2.exceptions
from jinja2 import Environment, PackageLoader, Template, select_autoescape


class TemplateNotFoundError(Exception):
    """Exception raise when a template was not found"""


def load_template(name: str) -> Template:
    """Load the template with the given name"""
    loader = PackageLoader("gbp_webhook_tts")
    env = Environment(loader=loader, autoescape=select_autoescape(["html", "xml"]))

    try:
        return env.get_template(name)
    except jinja2.exceptions.TemplateNotFound as error:
        raise TemplateNotFoundError(name, error.message) from error


def render_template(template: Template, context: dict[str, t.Any]) -> str:
    """Render the given Template given the context"""
    return template.render(**context)



"""Run tests for gbpcli"""

import argparse
import unittest


def main() -> None:
    """Program entry point"""
    args = parse_args()

    loader = unittest.TestLoader()
    loader.testNamePatterns = [f"*{pattern}*" for pattern in args.tests] or None
    tests = loader.discover("")
    verbosity = 2 if args.verbose else 1
    runner = unittest.TextTestRunner(verbosity=verbosity, failfast=args.failfast)
    test_result = runner.run(tests)

    raise SystemExit(int(not test_result.wasSuccessful()))


def parse_args() -> argparse.Namespace:
    """Parse command-line arguments"""
    parser = argparse.ArgumentParser()
    parser.add_argument("-f", "--failfast", action="store_true", default=False)
    parser.add_argument("-v", "--verbose", action="store_true", default=False)
    parser.add_argument("tests", nargs="*", default=[])

    return parser.parse_args()


if __name__ == "__main__":
    main()
